import * as i0 from '@angular/core';
import { Pipe, NgModule } from '@angular/core';
import { pinyin } from 'pinyin-pro';

const DefaultMatchOptions = {
    precision: 'first',
    continuous: false,
    space: 'ignore',
    lastPrecision: 'start',
};
const MAX_PINYIN_LENGTH = 6;
/**
 * @description: 检测汉语字符串和拼音是否匹配
 * @param {string} text 汉语字符串
 * @param {string} pinyin 拼音，支持各种缩写形式
 * @param {MatchOptions=} options 配置项
 * @return {Array | null} 若匹配成功，返回 text 中匹配成功的下标数组；若匹配失败，返回 null
 */
const matchOverride = (text, pinyin, options) => {
    if (options?.precision === 'any') {
        options.lastPrecision = 'any';
    }
    const completeOptions = {
        ...DefaultMatchOptions,
        ...(options || {}),
    };
    // 移除空格
    if (completeOptions.space === 'ignore') {
        pinyin = pinyin.replace(/\s/g, '');
    }
    const result = options?.precision === 'any'
        ? matchAny(text, pinyin, completeOptions)
        : matchAboveStart(text, pinyin, completeOptions);
    return result;
};
// /**
//  * @description: 检测汉语字符串和拼音是否匹配
//  * @param {string} text 汉语字符串
//  * @param {string} pinyin 拼音，支持各种缩写形式
//  * @return {Array | null} 若匹配成功，返回拼音在汉字中的下标数组；若匹配失败，返回 null
//  */
// export const matchOverride = (text: string, pinyin: string) => {
//   const result = [];
//   let currentPinyin = pinyin;
//   let i = 0;
//   // 是否为部分整体匹配
//   let lowerWords = text.toLowerCase();
//   let lowerPinyin = pinyin.toLowerCase();
//   if (lowerWords.includes(lowerPinyin)) {
//     i = lowerWords.indexOf(lowerPinyin);
//   }
//   for (; i < text.length; i++) {
//     // 是否为一致匹配
//     let x = lowerWords[i];
//     let y = currentPinyin[0]?.toLowerCase();
//     if (x === y) {
//       currentPinyin = currentPinyin.slice(1);
//       result.push(i);
//       continue;
//     }
//     // 当前字的多音字拼音
//     const ps = _pinyin(text[i], {
//       toneType: 'none',
//       multiple: true,
//       type: 'array',
//     });
//     let currentLength = 0;
//     ps.forEach((p) => {
//       const length = getMatchLength(p, currentPinyin);
//       if (length > currentLength) {
//         currentLength = length;
//       }
//     });
//     if (currentLength) {
//       currentPinyin = currentPinyin.slice(currentLength);
//       result.push(i);
//     }
//     if (!currentPinyin) {
//       break;
//     }
//   }
//   return result.length && !currentPinyin ? result : null;
// };
// 检测两个拼音最大的匹配长度
const getMatchLength = (pinyin1, pinyin2) => {
    let length = 0;
    for (let i = 0; i < pinyin1.length; i++) {
        if (pinyin1[i] === pinyin2[length]) {
            length++;
        }
    }
    return length;
};
const matchAny = (text, pinyin$1, options) => {
    let result = [];
    text = text.toLowerCase();
    pinyin$1 = pinyin$1.toLowerCase();
    let i = 0;
    const matchIndex = text.indexOf(pinyin$1);
    if (matchIndex !== -1)
        i = matchIndex;
    for (; i < text.length; i++) {
        // 空格字符
        if (options.space === 'ignore' && text[i] === ' ') {
            result.push(i);
            continue;
        }
        // 是否为一致匹配
        if (text[i] === pinyin$1[0]) {
            pinyin$1 = pinyin$1.slice(1);
            result.push(i);
            continue;
        }
        // 当前字的多音字拼音
        const ps = pinyin(text[i], {
            toneType: 'none',
            multiple: true,
            type: 'array',
        });
        let currentLength = 0;
        ps.forEach((p) => {
            const length = getMatchLength(p, pinyin$1);
            if (length > currentLength) {
                currentLength = length;
            }
        });
        if (currentLength) {
            pinyin$1 = pinyin$1.slice(currentLength);
            result.push(i);
        }
        if (!pinyin$1) {
            break;
        }
    }
    // 未匹配完
    if (pinyin$1) {
        return null;
    }
    // 是否连续
    if (options.continuous) {
        const _result = result;
        const isNotContinuous = result.some((val, index) => index > 0 && val !== _result[index - 1] + 1);
        if (isNotContinuous) {
            return null;
        }
    }
    if (options.space === 'ignore') {
        result = result.filter((i) => text[i] !== ' ');
    }
    return result.length ? result : null;
};
const matchAboveStart = (text, pinyin$1, options) => {
    text = text.toLowerCase();
    pinyin$1 = pinyin$1.toLowerCase();
    if (text.includes(pinyin$1))
        return matchAny(text, pinyin$1, { ...options, precision: 'any' });
    const words = text.split('');
    // 二维数组 dp[i][j]，i 表示遍历到的 text 索引+1, j 表示遍历到的 pinyin 的索引+1
    const dp = Array(words.length + 1);
    // 使用哨兵初始化 dp
    for (let i = 0; i < dp.length; i++) {
        dp[i] = Array(pinyin$1.length + 1);
        dp[i][0] = [];
    }
    for (let i = 0; i < dp[0].length; i++) {
        dp[0][i] = [];
    }
    // 动态规划匹配
    for (let i = 1; i < dp.length; i++) {
        // options.continuous 为 false 或 options.space 为 ignore 且当前为空格时，第 i 个字可以不参与匹配
        if (!options.continuous ||
            (options.space == 'ignore' && text[i - 1] === ' ')) {
            for (let j = 1; j <= pinyin$1.length; j++) {
                dp[i][j - 1] = dp[i - 1][j - 1];
            }
        }
        // 第 i 个字参与匹配
        for (let j = 1; j <= pinyin$1.length; j++) {
            if (!dp[i - 1][j - 1]) {
                // 第 i - 1 已经匹配失败，停止向后匹配
                continue;
            }
            else if (j !== 1 && !dp[i - 1][j - 1].length) {
                // 非开头且前面的字符未匹配完成，停止向后匹配
                continue;
            }
            else {
                const muls = pinyin(text[i - 1], {
                    type: 'array',
                    toneType: 'none',
                    multiple: true,
                });
                // 非中文匹配
                if (text[i - 1] === pinyin$1[j - 1]) {
                    const matches = [...dp[i - 1][j - 1], i - 1];
                    // 记录最长的可匹配下标数组
                    if (!dp[i][j] || matches.length > dp[i][j].length) {
                        dp[i][j] = matches;
                    }
                    // pinyin 参数完全匹配完成，记录结果
                    if (j === pinyin$1.length) {
                        return dp[i][j];
                    }
                }
                // 剩余长度小于等于 MAX_PINYIN_LENGTH(6) 时，有可能是最后一个拼音了
                if (pinyin$1.length - j <= MAX_PINYIN_LENGTH) {
                    // lastPrecision 参数处理
                    const last = muls.some((py) => {
                        if (options.lastPrecision === 'any') {
                            return py.includes(pinyin$1.slice(j - 1, pinyin$1.length));
                        }
                        if (options.lastPrecision === 'start') {
                            return py.startsWith(pinyin$1.slice(j - 1, pinyin$1.length));
                        }
                        if (options.lastPrecision === 'first') {
                            return py[0] === pinyin$1.slice(j - 1, pinyin$1.length);
                        }
                        if (options.lastPrecision === 'every') {
                            return py === pinyin$1.slice(j - 1, pinyin$1.length);
                        }
                        return false;
                    });
                    if (last) {
                        return [...dp[i - 1][j - 1], i - 1];
                    }
                }
                const precision = options.precision;
                // precision 为 start 时，匹配开头
                if (precision === 'start') {
                    muls.forEach((py) => {
                        let end = j;
                        const matches = [...dp[i - 1][j - 1], i - 1];
                        while (end <= pinyin$1.length &&
                            py.startsWith(pinyin$1.slice(j - 1, end))) {
                            if (!dp[i][end] || matches.length > dp[i][end].length) {
                                dp[i][end] = matches;
                            }
                            end++;
                        }
                    });
                }
                // precision 为 first 时，匹配首字母
                if (precision === 'first') {
                    if (muls.some((py) => py[0] === pinyin$1[j - 1])) {
                        const matches = [...dp[i - 1][j - 1], i - 1];
                        // 记录最长的可匹配下标数组
                        if (!dp[i][j] || matches.length > dp[i][j].length) {
                            dp[i][j] = matches;
                        }
                    }
                }
                // 匹配当前汉字的完整拼音
                const completeMatch = muls.find((py) => py === pinyin$1.slice(j - 1, j - 1 + py.length));
                if (completeMatch) {
                    const matches = [...dp[i - 1][j - 1], i - 1];
                    const endIndex = j - 1 + completeMatch.length;
                    // 记录最长的可匹配下标数组
                    if (!dp[i][endIndex] || matches.length > dp[i][endIndex].length) {
                        dp[i][endIndex] = matches;
                    }
                }
            }
        }
    }
    return null;
};

const escape = (s) => s.replace(/[-\/\\^$*+?.()|[\]{}]/g, '\\$&');
const replaceStartPattern = (s) => new RegExp(`^${s}`);
class HighlightPipe {
    /**
     * @name transform
     * @param value {string}
     * @param input {string}
     * @param pinyinMode {boolean} if enable pinyin parsing mode
     */
    transform(value, input, pinyinMode = false) {
        const trimmedInput = input?.trim();
        if (trimmedInput === null ||
            trimmedInput === undefined ||
            trimmedInput === '') {
            return value;
        }
        try {
            if (pinyinMode) {
                const m = matchOverride(value, input, { precision: 'start' });
                if (m !== null) {
                    const pinyinArr = pinyin(value, {
                        nonZh: 'consecutive',
                        toneType: 'none',
                        v: true,
                        type: 'array',
                    });
                    if (pinyinArr.length) {
                        const firstCharArr = pinyin(value, {
                            pattern: 'first',
                            nonZh: 'consecutive',
                            toneType: 'none',
                            v: true,
                            type: 'array',
                        });
                        let newInputArr = [];
                        m.forEach((mi, i) => {
                            let regex = new RegExp(replaceStartPattern(pinyinArr[i]));
                            let newInput = input.replace(regex, '');
                            if (newInput === input) {
                                regex = new RegExp(replaceStartPattern(firstCharArr[i]));
                                newInput = input.replace(regex, '');
                            }
                            newInputArr.push(value[mi]);
                            input = newInput;
                        });
                        input = newInputArr.join('');
                    }
                }
            }
            const regex = new RegExp(`(${escape(input)})`, 'i');
            return value.replace(regex, '<b>$1</b>');
        }
        catch (e) {
            return value;
        }
    }
}
HighlightPipe.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.2", ngImport: i0, type: HighlightPipe, deps: [], target: i0.ɵɵFactoryTarget.Pipe });
HighlightPipe.ɵpipe = i0.ɵɵngDeclarePipe({ minVersion: "14.0.0", version: "15.2.2", ngImport: i0, type: HighlightPipe, name: "highlight" });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.2", ngImport: i0, type: HighlightPipe, decorators: [{
            type: Pipe,
            args: [{
                    name: 'highlight',
                }]
        }] });

class OdHighlightPipeModule {
}
OdHighlightPipeModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.2", ngImport: i0, type: OdHighlightPipeModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
OdHighlightPipeModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "15.2.2", ngImport: i0, type: OdHighlightPipeModule, declarations: [HighlightPipe], exports: [HighlightPipe] });
OdHighlightPipeModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "15.2.2", ngImport: i0, type: OdHighlightPipeModule });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.2", ngImport: i0, type: OdHighlightPipeModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [HighlightPipe],
                    imports: [],
                    exports: [HighlightPipe],
                }]
        }] });

/*
 * Public API Surface of @linch90/od-highlight-pipe
 */

/**
 * Generated bundle index. Do not edit.
 */

export { HighlightPipe, OdHighlightPipeModule };
//# sourceMappingURL=linch90-od-highlight-pipe.mjs.map
